@@ -3,19 +3,19 @@
 
 use cranelift_codegen::Context;
 use cranelift_codegen::MachSrcLoc;
-use wasmer_compiler::wasmparser::Range;
+use std::ops::Range;
 use wasmer_types::{FunctionAddressMap, InstructionAddressMap, SourceLoc};
 
 pub fn get_function_address_map(
     context: &Context,
-    range: Range,
+    range: Range<usize>,
     body_len: usize,
 ) -> FunctionAddressMap {
     let mut instructions = Vec::new();
 
     // New-style backend: we have a `MachCompileResult` that will give us `MachSrcLoc` mapping
     // tuples.
-    let mcr = context.mach_compile_result.as_ref().unwrap();
+    let mcr = context.compiled_code().unwrap();
     for &MachSrcLoc { start, end, loc } in mcr.buffer.get_srclocs_sorted() {
         instructions.push(InstructionAddressMap {
             srcloc: SourceLoc::new(loc.bits()),
