@@ -23,11 +23,10 @@ use inkwell::{
     AddressSpace,
 };
 use std::collections::{hash_map::Entry, HashMap};
-use wasmer_compiler::CompileError;
 use wasmer_types::entity::{EntityRef, PrimaryMap};
 use wasmer_types::{
-    FunctionIndex, FunctionType as FuncType, GlobalIndex, LocalFunctionIndex, MemoryIndex,
-    ModuleInfo as WasmerCompilerModule, Mutability, SignatureIndex, TableIndex, Type,
+    CompileError, FunctionIndex, FunctionType as FuncType, GlobalIndex, LocalFunctionIndex,
+    MemoryIndex, ModuleInfo as WasmerCompilerModule, Mutability, SignatureIndex, TableIndex, Type,
 };
 use wasmer_vm::{MemoryStyle, TrapCode, VMBuiltinFunctionIndex, VMOffsets};
 
@@ -41,8 +40,8 @@ pub fn type_to_llvm_ptr<'ctx>(
         Type::F32 => Ok(intrinsics.f32_ptr_ty),
         Type::F64 => Ok(intrinsics.f64_ptr_ty),
         Type::V128 => Ok(intrinsics.i128_ptr_ty),
-        Type::FuncRef => Ok(intrinsics.funcref_ty.ptr_type(AddressSpace::Generic)),
-        Type::ExternRef => Ok(intrinsics.externref_ty.ptr_type(AddressSpace::Generic)),
+        Type::FuncRef => Ok(intrinsics.funcref_ty.ptr_type(AddressSpace::default())),
+        Type::ExternRef => Ok(intrinsics.externref_ty.ptr_type(AddressSpace::default())),
     }
 }
 
@@ -241,6 +240,12 @@ pub struct Intrinsics<'ctx> {
     pub imported_memory_copy: FunctionValue<'ctx>,
     pub memory_fill: FunctionValue<'ctx>,
     pub imported_memory_fill: FunctionValue<'ctx>,
+    pub memory_wait32: FunctionValue<'ctx>,
+    pub imported_memory_wait32: FunctionValue<'ctx>,
+    pub memory_wait64: FunctionValue<'ctx>,
+    pub imported_memory_wait64: FunctionValue<'ctx>,
+    pub memory_notify: FunctionValue<'ctx>,
+    pub imported_memory_notify: FunctionValue<'ctx>,
 
     pub throw_trap: FunctionValue<'ctx>,
 
@@ -257,6 +262,12 @@ pub struct Intrinsics<'ctx> {
     pub imported_memory32_grow_ptr_ty: PointerType<'ctx>,
     pub memory32_size_ptr_ty: PointerType<'ctx>,
     pub imported_memory32_size_ptr_ty: PointerType<'ctx>,
+    pub memory32_wait32_ptr_ty: PointerType<'ctx>,
+    pub imported_memory32_wait32_ptr_ty: PointerType<'ctx>,
+    pub memory32_wait64_ptr_ty: PointerType<'ctx>,
+    pub imported_memory32_wait64_ptr_ty: PointerType<'ctx>,
+    pub memory32_notify_ptr_ty: PointerType<'ctx>,
+    pub imported_memory32_notify_ptr_ty: PointerType<'ctx>,
 
     // Pointer to the VM.
     pub ctx_ptr_ty: PointerType<'ctx>,
@@ -293,14 +304,14 @@ impl<'ctx> Intrinsics<'ctx> {
         let f64x2_ty = f64_ty.vec_type(2);
         let i32x8_ty = i32_ty.vec_type(8);
 
-        let i8_ptr_ty = i8_ty.ptr_type(AddressSpace::Generic);
-        let i16_ptr_ty = i16_ty.ptr_type(AddressSpace::Generic);
-        let i32_ptr_ty = i32_ty.ptr_type(AddressSpace::Generic);
-        let i64_ptr_ty = i64_ty.ptr_type(AddressSpace::Generic);
-        let i128_ptr_ty = i128_ty.ptr_type(AddressSpace::Generic);
-        let isize_ptr_ty = isize_ty.ptr_type(AddressSpace::Generic);
-        let f32_ptr_ty = f32_ty.ptr_type(AddressSpace::Generic);
-        let f64_ptr_ty = f64_ty.ptr_type(AddressSpace::Generic);
+        let i8_ptr_ty = i8_ty.ptr_type(AddressSpace::default());
+        let i16_ptr_ty = i16_ty.ptr_type(AddressSpace::default());
+        let i32_ptr_ty = i32_ty.ptr_type(AddressSpace::default());
+        let i64_ptr_ty = i64_ty.ptr_type(AddressSpace::default());
+        let i128_ptr_ty = i128_ty.ptr_type(AddressSpace::default());
+        let isize_ptr_ty = isize_ty.ptr_type(AddressSpace::default());
+        let f32_ptr_ty = f32_ty.ptr_type(AddressSpace::default());
+        let f64_ptr_ty = f64_ty.ptr_type(AddressSpace::default());
 
         let i1_zero = i1_ty.const_int(0, false);
         let i8_zero = i8_ty.const_int(0, false);
@@ -347,7 +358,7 @@ impl<'ctx> Intrinsics<'ctx> {
         let md_ty_basic_md: BasicMetadataTypeEnum = md_ty.into();
 
         let ctx_ty = i8_ty;
-        let ctx_ptr_ty = ctx_ty.ptr_type(AddressSpace::Generic);
+        let ctx_ptr_ty = ctx_ty.ptr_type(AddressSpace::default());
         let ctx_ptr_ty_basic = ctx_ptr_ty.as_basic_type_enum();
         let ctx_ptr_ty_basic_md: BasicMetadataTypeEnum = ctx_ptr_ty.into();
 
@@ -357,7 +368,7 @@ impl<'ctx> Intrinsics<'ctx> {
             &[i8_ptr_ty_basic, sigindex_ty.into(), ctx_ptr_ty_basic],
             false,
         );
-        let funcref_ty = anyfunc_ty.ptr_type(AddressSpace::Generic);
+        let funcref_ty = anyfunc_ty.ptr_type(AddressSpace::default());
         let externref_ty = funcref_ty;
         let anyref_ty = i8_ptr_ty;
         let anyref_ty_basic_md: BasicMetadataTypeEnum = anyref_ty.into();
@@ -703,7 +714,7 @@ impl<'ctx> Intrinsics<'ctx> {
             ),
             readonly: context
                 .create_enum_attribute(Attribute::get_named_enum_kind_id("readonly"), 0),
-            stack_probe: context.create_string_attribute("probe-stack", "wasmer_vm_probestack"),
+            stack_probe: context.create_string_attribute("probe-stack", "inline-asm"),
 
             void_ty,
             i1_ty,
@@ -1008,16 +1019,88 @@ impl<'ctx> Intrinsics<'ctx> {
                 void_ty.fn_type(&[i32_ty_basic_md], false),
                 None,
             ),
+            memory_wait32: module.add_function(
+                "wasmer_vm_memory32_atomic_wait32",
+                i32_ty.fn_type(
+                    &[
+                        ctx_ptr_ty_basic_md,
+                        i32_ty_basic_md,
+                        i32_ty_basic_md,
+                        i32_ty_basic_md,
+                        i64_ty_basic_md,
+                    ],
+                    false,
+                ),
+                None,
+            ),
+            imported_memory_wait32: module.add_function(
+                "wasmer_vm_imported_memory32_atomic_wait32",
+                i32_ty.fn_type(
+                    &[
+                        ctx_ptr_ty_basic_md,
+                        i32_ty_basic_md,
+                        i32_ty_basic_md,
+                        i32_ty_basic_md,
+                        i64_ty_basic_md,
+                    ],
+                    false,
+                ),
+                None,
+            ),
+            memory_wait64: module.add_function(
+                "wasmer_vm_memory32_atomic_wait64",
+                i32_ty.fn_type(
+                    &[
+                        ctx_ptr_ty_basic_md,
+                        i32_ty_basic_md,
+                        i32_ty_basic_md,
+                        i64_ty_basic_md,
+                        i64_ty_basic_md,
+                    ],
+                    false,
+                ),
+                None,
+            ),
+            imported_memory_wait64: module.add_function(
+                "wasmer_vm_imported_memory32_atomic_wait64",
+                i32_ty.fn_type(
+                    &[
+                        ctx_ptr_ty_basic_md,
+                        i32_ty_basic_md,
+                        i32_ty_basic_md,
+                        i64_ty_basic_md,
+                        i64_ty_basic_md,
+                    ],
+                    false,
+                ),
+                None,
+            ),
+            memory_notify: module.add_function(
+                "wasmer_vm_memory32_atomic_notify",
+                i32_ty.fn_type(
+                    &[ctx_ptr_ty_basic_md, i32_ty_basic_md, i32_ty_basic_md],
+                    false,
+                ),
+                None,
+            ),
+            imported_memory_notify: module.add_function(
+                "wasmer_vm_imported_memory32_atomic_notify",
+                i32_ty.fn_type(
+                    &[ctx_ptr_ty_basic_md, i32_ty_basic_md, i32_ty_basic_md],
+                    false,
+                ),
+                None,
+            ),
 
             vmfunction_import_ptr_ty: context
                 .struct_type(&[i8_ptr_ty_basic, i8_ptr_ty_basic], false)
-                .ptr_type(AddressSpace::Generic),
+                .ptr_type(AddressSpace::default()),
             vmfunction_import_body_element: 0,
             vmfunction_import_vmctx_element: 1,
 
             vmmemory_definition_ptr_ty: context
                 .struct_type(&[i8_ptr_ty_basic, isize_ty.into()], false)
-                .ptr_type(AddressSpace::Generic),
+                .ptr_type(AddressSpace::default()),
             vmmemory_definition_base_element: 0,
             vmmemory_definition_current_length_element: 1,
 
@@ -1026,19 +1109,89 @@ impl<'ctx> Intrinsics<'ctx> {
                     &[ctx_ptr_ty_basic_md, i32_ty_basic_md, i32_ty_basic_md],
                     false,
                 )
-                .ptr_type(AddressSpace::Generic),
+                .ptr_type(AddressSpace::default()),
             imported_memory32_grow_ptr_ty: i32_ty
                 .fn_type(
                     &[ctx_ptr_ty_basic_md, i32_ty_basic_md, i32_ty_basic_md],
                     false,
                 )
-                .ptr_type(AddressSpace::Generic),
+                .ptr_type(AddressSpace::default()),
             memory32_size_ptr_ty: i32_ty
                 .fn_type(&[ctx_ptr_ty_basic_md, i32_ty_basic_md], false)
-                .ptr_type(AddressSpace::Generic),
+                .ptr_type(AddressSpace::default()),
             imported_memory32_size_ptr_ty: i32_ty
                 .fn_type(&[ctx_ptr_ty_basic_md, i32_ty_basic_md], false)
-                .ptr_type(AddressSpace::Generic),
+                .ptr_type(AddressSpace::default()),
+            memory32_wait32_ptr_ty: i32_ty
+                .fn_type(
+                    &[
+                        ctx_ptr_ty_basic_md,
+                        i32_ty_basic_md,
+                        i32_ty_basic_md,
+                        i32_ty_basic_md,
+                        i64_ty_basic_md,
+                    ],
+                    false,
+                )
+                .ptr_type(AddressSpace::default()),
+            imported_memory32_wait32_ptr_ty: i32_ty
+                .fn_type(
+                    &[
+                        ctx_ptr_ty_basic_md,
+                        i32_ty_basic_md,
+                        i32_ty_basic_md,
+                        i32_ty_basic_md,
+                        i64_ty_basic_md,
+                    ],
+                    false,
+                )
+                .ptr_type(AddressSpace::default()),
+            memory32_wait64_ptr_ty: i32_ty
+                .fn_type(
+                    &[
+                        ctx_ptr_ty_basic_md,
+                        i32_ty_basic_md,
+                        i32_ty_basic_md,
+                        i64_ty_basic_md,
+                        i64_ty_basic_md,
+                    ],
+                    false,
+                )
+                .ptr_type(AddressSpace::default()),
+            imported_memory32_wait64_ptr_ty: i32_ty
+                .fn_type(
+                    &[
+                        ctx_ptr_ty_basic_md,
+                        i32_ty_basic_md,
+                        i32_ty_basic_md,
+                        i64_ty_basic_md,
+                        i64_ty_basic_md,
+                    ],
+                    false,
+                )
+                .ptr_type(AddressSpace::default()),
+            memory32_notify_ptr_ty: i32_ty
+                .fn_type(
+                    &[
+                        ctx_ptr_ty_basic_md,
+                        i32_ty_basic_md,
+                        i32_ty_basic_md,
+                        i32_ty_basic_md,
+                    ],
+                    false,
+                )
+                .ptr_type(AddressSpace::default()),
+            imported_memory32_notify_ptr_ty: i32_ty
+                .fn_type(
+                    &[
+                        ctx_ptr_ty_basic_md,
+                        i32_ty_basic_md,
+                        i32_ty_basic_md,
+                        i32_ty_basic_md,
+                    ],
+                    false,
+                )
+                .ptr_type(AddressSpace::default()),
 
             ctx_ptr_ty,
         };
@@ -1126,7 +1279,7 @@ impl<'ctx, 'a> CtxType<'ctx, 'a> {
             cached_memory_size: HashMap::new(),
 
             // TODO: pointer width
-            offsets: VMOffsets::new(8, &wasm_module),
+            offsets: VMOffsets::new(8, wasm_module),
         }
     }
 
@@ -1163,7 +1316,7 @@ impl<'ctx, 'a> CtxType<'ctx, 'a> {
                     let memory_definition_ptr_ptr = cache_builder
                         .build_bitcast(
                             memory_definition_ptr_ptr,
-                            intrinsics.i8_ptr_ty.ptr_type(AddressSpace::Generic),
+                            intrinsics.i8_ptr_ty.ptr_type(AddressSpace::default()),
                             "",
                         )
                         .into_pointer_value();
@@ -1247,7 +1400,7 @@ impl<'ctx, 'a> CtxType<'ctx, 'a> {
                     let ptr_to_base_ptr = cache_builder
                         .build_bitcast(
                             ptr_to_base_ptr,
-                            intrinsics.i8_ptr_ty.ptr_type(AddressSpace::Generic),
+                            intrinsics.i8_ptr_ty.ptr_type(AddressSpace::default()),
                             "",
                         )
                         .into_pointer_value();
@@ -1273,7 +1426,7 @@ impl<'ctx, 'a> CtxType<'ctx, 'a> {
                     let definition_ptr_ptr = cache_builder
                         .build_bitcast(
                             definition_ptr_ptr,
-                            intrinsics.i8_ptr_ty.ptr_type(AddressSpace::Generic),
+                            intrinsics.i8_ptr_ty.ptr_type(AddressSpace::default()),
                             "",
                         )
                         .into_pointer_value();
@@ -1295,7 +1448,7 @@ impl<'ctx, 'a> CtxType<'ctx, 'a> {
                     let ptr_to_base_ptr = cache_builder
                         .build_bitcast(
                             ptr_to_base_ptr,
-                            intrinsics.i8_ptr_ty.ptr_type(AddressSpace::Generic),
+                            intrinsics.i8_ptr_ty.ptr_type(AddressSpace::default()),
                             "",
                         )
                         .into_pointer_value();
@@ -1417,7 +1570,7 @@ impl<'ctx, 'a> CtxType<'ctx, 'a> {
                     let global_ptr_ptr = cache_builder
                         .build_bitcast(
                             global_ptr_ptr,
-                            intrinsics.i32_ptr_ty.ptr_type(AddressSpace::Generic),
+                            intrinsics.i32_ptr_ty.ptr_type(AddressSpace::default()),
                             "",
                         )
                         .into_pointer_value();
@@ -1435,7 +1588,7 @@ impl<'ctx, 'a> CtxType<'ctx, 'a> {
                 let global_ptr = cache_builder
                     .build_bitcast(
                         global_ptr,
-                        type_to_llvm_ptr(&intrinsics, global_value_type)?,
+                        type_to_llvm_ptr(intrinsics, global_value_type)?,
                         "",
                     )
                     .into_pointer_value();
@@ -1478,6 +1631,7 @@ impl<'ctx, 'a> CtxType<'ctx, 'a> {
         }
     }
 
+    #[allow(clippy::too_many_arguments)]
     pub fn local_func(
         &mut self,
         _local_function_index: LocalFunctionIndex,
@@ -1556,7 +1710,11 @@ impl<'ctx, 'a> CtxType<'ctx, 'a> {
                     .unwrap();
                 let body_ptr = cache_builder.build_load(body_ptr_ptr, "");
                 let body_ptr = cache_builder
-                    .build_bitcast(body_ptr, llvm_func_type.ptr_type(AddressSpace::Generic), "")
+                    .build_bitcast(
+                        body_ptr,
+                        llvm_func_type.ptr_type(AddressSpace::default()),
+                        "",
+                    )
                     .into_pointer_value();
                 let vmctx_ptr_ptr = cache_builder
                     .build_struct_gep(
@@ -1606,7 +1764,7 @@ impl<'ctx, 'a> CtxType<'ctx, 'a> {
             let grow_fn_ptr_ptr = cache_builder
                 .build_bitcast(
                     grow_fn_ptr_ptr,
-                    grow_fn_ty.ptr_type(AddressSpace::Generic),
+                    grow_fn_ty.ptr_type(AddressSpace::default()),
                     "",
                 )
                 .into_pointer_value();
@@ -1647,7 +1805,133 @@ impl<'ctx, 'a> CtxType<'ctx, 'a> {
             let size_fn_ptr_ptr = cache_builder
                 .build_bitcast(
                     size_fn_ptr_ptr,
-                    size_fn_ty.ptr_type(AddressSpace::Generic),
+                    size_fn_ty.ptr_type(AddressSpace::default()),
+                    "",
+                )
+                .into_pointer_value();
+
+            cache_builder
+                .build_load(size_fn_ptr_ptr, "")
+                .into_pointer_value()
+        })
+    }
+
+    pub fn memory_wait32(
+        &mut self,
+        memory_index: MemoryIndex,
+        intrinsics: &Intrinsics<'ctx>,
+    ) -> PointerValue<'ctx> {
+        let (cached_memory_size, wasm_module, offsets, cache_builder, ctx_ptr_value) = (
+            &mut self.cached_memory_size,
+            &self.wasm_module,
+            &self.offsets,
+            &self.cache_builder,
+            &self.ctx_ptr_value,
+        );
+        *cached_memory_size.entry(memory_index).or_insert_with(|| {
+            let (size_fn, size_fn_ty) = if wasm_module.local_memory_index(memory_index).is_some() {
+                (
+                    VMBuiltinFunctionIndex::get_memory_atomic_wait32_index(),
+                    intrinsics.memory32_wait32_ptr_ty,
+                )
+            } else {
+                (
+                    VMBuiltinFunctionIndex::get_imported_memory_atomic_wait32_index(),
+                    intrinsics.imported_memory32_wait32_ptr_ty,
+                )
+            };
+            let offset = offsets.vmctx_builtin_function(size_fn);
+            let offset = intrinsics.i32_ty.const_int(offset.into(), false);
+            let size_fn_ptr_ptr = unsafe { cache_builder.build_gep(*ctx_ptr_value, &[offset], "") };
+
+            let size_fn_ptr_ptr = cache_builder
+                .build_bitcast(
+                    size_fn_ptr_ptr,
+                    size_fn_ty.ptr_type(AddressSpace::default()),
+                    "",
+                )
+                .into_pointer_value();
+
+            cache_builder
+                .build_load(size_fn_ptr_ptr, "")
+                .into_pointer_value()
+        })
+    }
+
+    pub fn memory_wait64(
+        &mut self,
+        memory_index: MemoryIndex,
+        intrinsics: &Intrinsics<'ctx>,
+    ) -> PointerValue<'ctx> {
+        let (cached_memory_size, wasm_module, offsets, cache_builder, ctx_ptr_value) = (
+            &mut self.cached_memory_size,
+            &self.wasm_module,
+            &self.offsets,
+            &self.cache_builder,
+            &self.ctx_ptr_value,
+        );
+        *cached_memory_size.entry(memory_index).or_insert_with(|| {
+            let (size_fn, size_fn_ty) = if wasm_module.local_memory_index(memory_index).is_some() {
+                (
+                    VMBuiltinFunctionIndex::get_memory_atomic_wait64_index(),
+                    intrinsics.memory32_wait64_ptr_ty,
+                )
+            } else {
+                (
+                    VMBuiltinFunctionIndex::get_imported_memory_atomic_wait64_index(),
+                    intrinsics.imported_memory32_wait64_ptr_ty,
+                )
+            };
+            let offset = offsets.vmctx_builtin_function(size_fn);
+            let offset = intrinsics.i32_ty.const_int(offset.into(), false);
+            let size_fn_ptr_ptr = unsafe { cache_builder.build_gep(*ctx_ptr_value, &[offset], "") };
+
+            let size_fn_ptr_ptr = cache_builder
+                .build_bitcast(
+                    size_fn_ptr_ptr,
+                    size_fn_ty.ptr_type(AddressSpace::default()),
+                    "",
+                )
+                .into_pointer_value();
+
+            cache_builder
+                .build_load(size_fn_ptr_ptr, "")
+                .into_pointer_value()
+        })
+    }
+
+    pub fn memory_notify(
+        &mut self,
+        memory_index: MemoryIndex,
+        intrinsics: &Intrinsics<'ctx>,
+    ) -> PointerValue<'ctx> {
+        let (cached_memory_size, wasm_module, offsets, cache_builder, ctx_ptr_value) = (
+            &mut self.cached_memory_size,
+            &self.wasm_module,
+            &self.offsets,
+            &self.cache_builder,
+            &self.ctx_ptr_value,
+        );
+        *cached_memory_size.entry(memory_index).or_insert_with(|| {
+            let (size_fn, size_fn_ty) = if wasm_module.local_memory_index(memory_index).is_some() {
+                (
+                    VMBuiltinFunctionIndex::get_memory_atomic_notify_index(),
+                    intrinsics.memory32_notify_ptr_ty,
+                )
+            } else {
+                (
+                    VMBuiltinFunctionIndex::get_imported_memory_atomic_notify_index(),
+                    intrinsics.imported_memory32_notify_ptr_ty,
+                )
+            };
+            let offset = offsets.vmctx_builtin_function(size_fn);
+            let offset = intrinsics.i32_ty.const_int(offset.into(), false);
+            let size_fn_ptr_ptr = unsafe { cache_builder.build_gep(*ctx_ptr_value, &[offset], "") };
+
+            let size_fn_ptr_ptr = cache_builder
+                .build_bitcast(
+                    size_fn_ptr_ptr,
+                    size_fn_ty.ptr_type(AddressSpace::default()),
                     "",
                 )
                 .into_pointer_value();
@@ -1686,13 +1970,6 @@ pub fn tbaa_label<'ctx>(
 
     let context = module.get_context();
 
-    // TODO: ContextRef can't return us the lifetime from module through Deref.
-    // This could be fixed once generic_associated_types is stable.
-    let context = {
-        let context2 = &*context;
-        unsafe { std::mem::transmute::<&Context, &'ctx Context>(context2) }
-    };
-
     // `!wasmer_tbaa_root = {}`, the TBAA root node for wasmer.
     let tbaa_root = module
         .get_global_metadata("wasmer_tbaa_root")
