@@ -1,7 +1,8 @@
 use crate::EmEnv;
+use wasmer::FunctionEnvMut;
 
 // __exit
-pub fn exit(_ctx: &EmEnv, value: i32) {
+pub fn exit(mut _ctx: FunctionEnvMut<EmEnv>, value: i32) {
     debug!("emscripten::exit {}", value);
     ::std::process::exit(value);
 }
