@@ -291,11 +291,22 @@ impl VirtualFile for FileHandle {
             let inode = fs.storage.get_mut(self.inode);
             match inode {
                 Some(inode) => {
+                    let metadata = Metadata {
+                        ft: crate::FileType {
+                            file: true,
+                            ..Default::default()
+                        },
+                        accessed: src.last_accessed(),
+                        created: src.created_time(),
+                        modified: src.last_modified(),
+                        len: src.size(),
+                    };
+
                     *inode = Node::CustomFile(CustomFileNode {
                         inode: inode.inode(),
                         name: inode.name().to_string_lossy().to_string().into(),
                         file: Mutex::new(Box::new(CopyOnWriteFile::new(src))),
-                        metadata: inode.metadata().clone(),
+                        metadata,
                     });
                     Ok(())
                 }
@@ -413,8 +424,8 @@ mod test_virtual_file {
         };
     }
 
-    #[test]
-    fn test_last_accessed() {
+    #[tokio::test]
+    async fn test_last_accessed() {
         let fs = FileSystem::default();
 
         let file = fs
@@ -442,8 +453,8 @@ mod test_virtual_file {
         );
     }
 
-    #[test]
-    fn test_last_modified() {
+    #[tokio::test]
+    async fn test_last_modified() {
         let fs = FileSystem::default();
 
         let file = fs
@@ -456,8 +467,8 @@ mod test_virtual_file {
         assert!(file.last_modified() > 0, "last modified time is not zero");
     }
 
-    #[test]
-    fn test_created_time() {
+    #[tokio::test]
+    async fn test_created_time() {
         let fs = FileSystem::default();
 
         let file = fs
@@ -483,8 +494,8 @@ mod test_virtual_file {
         );
     }
 
-    #[test]
-    fn test_size() {
+    #[tokio::test]
+    async fn test_size() {
         let fs = FileSystem::default();
 
         let file = fs
