@@ -11,7 +11,7 @@ mod traphandlers;
 pub use trap::Trap;
 pub use traphandlers::{
     catch_traps, on_host_stack, raise_lib_trap, raise_user_trap, set_stack_size,
-    wasmer_call_trampoline, TrapHandlerFn,
+    wasmer_call_trampoline, TrapHandlerFn, VMConfig,
 };
 pub use traphandlers::{init_traps, resume_panic};
 pub use wasmer_types::TrapCode;
