@@ -1,3 +1,5 @@
+#![cfg_attr(docsrs, feature(doc_cfg, doc_auto_cfg))]
+
 #[cfg(feature = "link_external_libs")]
 #[path = "link-ext.rs"]
 pub mod link_ext;
@@ -6,8 +8,11 @@ pub mod link_ext;
 pub use crate::link_ext::*;
 
 #[cfg(not(feature = "link_external_libs"))]
-use wasmer_wasi::{WasiFs, WasiInodes};
+use wasmer_wasix::fs::WasiFs;
 #[cfg(not(feature = "link_external_libs"))]
-pub fn initialize(_: &mut WasiInodes, _: &mut WasiFs) -> Result<(), String> {
+use wasmer_wasix::fs::WasiInodes;
+
+#[cfg(not(feature = "link_external_libs"))]
+pub fn initialize(_: &WasiInodes, _: &mut WasiFs) -> Result<(), String> {
     Err("wasi-experimental-io-devices has to be compiled with --features=\"link_external_libs\" (not enabled by default) for graphics I/O to work".to_string())
 }
