@@ -20,6 +20,7 @@ use crate::{
     capabilities::Capabilities,
     fs::{WasiFs, WasiFsRoot, WasiInodes},
     os::task::control_plane::{ControlPlaneConfig, ControlPlaneError, WasiControlPlane},
+    runtime::task_manager::InlineWaker,
     state::WasiState,
     syscalls::types::{__WASI_STDERR_FILENO, __WASI_STDIN_FILENO, __WASI_STDOUT_FILENO},
     RewindState, Runtime, WasiEnv, WasiError, WasiFunctionEnv, WasiRuntimeError,
@@ -708,7 +709,7 @@ impl WasiEnvBuilder {
         let runtime = self.runtime.unwrap_or_else(|| {
             #[cfg(feature = "sys-thread")]
             {
-                Arc::new(PluggableRuntime::new(Arc::new(crate::runtime::task_manager::tokio::TokioTaskManager::shared())))
+                Arc::new(PluggableRuntime::new(Arc::new(crate::runtime::task_manager::tokio::TokioTaskManager::default())))
             }
 
             #[cfg(not(feature = "sys-thread"))]
@@ -743,6 +744,7 @@ impl WasiEnvBuilder {
             thread: None,
             call_initialize: true,
             can_deep_sleep: false,
+            extra_tracing: true,
         };
 
         Ok(init)
@@ -793,6 +795,20 @@ impl WasiEnvBuilder {
 
     #[allow(clippy::result_large_err)]
     pub fn run_with_store(self, module: Module, store: &mut Store) -> Result<(), WasiRuntimeError> {
+        // If no handle or runtime exists then create one
+        #[cfg(feature = "sys-thread")]
+        let _guard = if tokio::runtime::Handle::try_current().is_err() {
+            let runtime = tokio::runtime::Builder::new_multi_thread()
+                .enable_all()
+                .build()
+                .unwrap();
+            Some(runtime)
+        } else {
+            None
+        };
+        #[cfg(feature = "sys-thread")]
+        let _guard = _guard.as_ref().map(|r| r.enter());
+
         if self.capabilites.threading.enable_asynchronous_threading {
             tracing::warn!(
                 "The enable_asynchronous_threading capability is enabled. Use WasiEnvBuilder::run_with_store_async() to avoid spurious errors.",
@@ -829,6 +845,20 @@ impl WasiEnvBuilder {
         module: Module,
         mut store: Store,
     ) -> Result<(), WasiRuntimeError> {
+        // If no handle or runtime exists then create one
+        #[cfg(feature = "sys-thread")]
+        let _guard = if tokio::runtime::Handle::try_current().is_err() {
+            let runtime = tokio::runtime::Builder::new_multi_thread()
+                .enable_all()
+                .build()
+                .unwrap();
+            Some(runtime)
+        } else {
+            None
+        };
+        #[cfg(feature = "sys-thread")]
+        let _guard = _guard.as_ref().map(|r| r.enter());
+
         let (_, env) = self.instantiate(module, &mut store)?;
 
         env.data(&store).thread.set_status_running();
@@ -840,15 +870,18 @@ impl WasiEnvBuilder {
         // The return value is passed synchronously and will block until the result
         // is returned this is because the main thread can go into a deep sleep and
         // exit the dedicated thread
-        let (tx, rx) = std::sync::mpsc::channel();
+        let (tx, mut rx) = tokio::sync::mpsc::unbounded_channel();
 
         tasks.task_dedicated(Box::new(move || {
             run_with_deep_sleep(store, None, env, tx);
         }))?;
 
-        let result = rx.recv().expect(
-            "main thread terminated without a result, this normally means a panic occurred",
-        );
+        let result = InlineWaker::block_on(rx.recv());
+        let result = result.unwrap_or_else(|| {
+            Err(WasiRuntimeError::Runtime(RuntimeError::new(
+                "main thread terminated without a result, this normally means a panic occurred",
+            )))
+        });
         let (result, exit_code) = wasi_exit_code(result);
 
         tracing::trace!(
@@ -892,7 +925,7 @@ fn run_with_deep_sleep(
     mut store: Store,
     rewind_state: Option<(RewindState, Bytes)>,
     env: WasiFunctionEnv,
-    sender: std::sync::mpsc::Sender<Result<(), WasiRuntimeError>>,
+    sender: tokio::sync::mpsc::UnboundedSender<Result<(), WasiRuntimeError>>,
 ) {
     if let Some((rewind_state, rewind_result)) = rewind_state {
         tracing::trace!("Rewinding");
@@ -952,7 +985,7 @@ fn handle_result(
     mut store: Store,
     env: WasiFunctionEnv,
     result: Result<Box<[wasmer::Value]>, RuntimeError>,
-    sender: std::sync::mpsc::Sender<Result<(), WasiRuntimeError>>,
+    sender: tokio::sync::mpsc::UnboundedSender<Result<(), WasiRuntimeError>>,
 ) {
     let result = match result.map_err(|e| e.downcast::<WasiError>()) {
         Err(Ok(WasiError::DeepSleep(work))) => {
@@ -981,7 +1014,7 @@ fn handle_result(
 
     let (result, exit_code) = wasi_exit_code(result);
     env.cleanup(&mut store, Some(exit_code));
-    let _ = sender.send(result);
+    sender.send(result).ok();
 }
 
 /// Builder for preopened directories.
@@ -1096,6 +1129,16 @@ mod test {
 
     #[test]
     fn env_var_errors() {
+        #[cfg(not(target_arch = "wasm32"))]
+        let runtime = tokio::runtime::Builder::new_multi_thread()
+            .enable_all()
+            .build()
+            .unwrap();
+        #[cfg(not(target_arch = "wasm32"))]
+        let handle = runtime.handle().clone();
+        #[cfg(not(target_arch = "wasm32"))]
+        let _guard = handle.enter();
+
         // `=` in the key is invalid.
         assert!(
             WasiEnv::builder("test_prog")
