@@ -1,7 +1,7 @@
-use std::mem::MaybeUninit;
+use std::{mem::MaybeUninit, task::Waker};
 
 use super::*;
-use crate::syscalls::*;
+use crate::{net::socket::TimeType, syscalls::*};
 
 /// ### `sock_send()`
 /// Send a message on a socket.
@@ -18,6 +18,17 @@ use crate::syscalls::*;
 /// Number of bytes transmitted.
 #[instrument(level = "trace", skip_all, fields(%sock, nsent = field::Empty), ret, err)]
 pub fn sock_send<M: MemorySize>(
+    ctx: FunctionEnvMut<'_, WasiEnv>,
+    sock: WasiFd,
+    si_data: WasmPtr<__wasi_ciovec_t<M>, M>,
+    si_data_len: M::Offset,
+    si_flags: SiFlags,
+    ret_data_len: WasmPtr<M::Offset, M>,
+) -> Result<Errno, WasiError> {
+    sock_send_internal(ctx, sock, si_data, si_data_len, si_flags, ret_data_len)
+}
+
+pub(super) fn sock_send_internal<M: MemorySize>(
     mut ctx: FunctionEnvMut<'_, WasiEnv>,
     sock: WasiFd,
     si_data: WasmPtr<__wasi_ciovec_t<M>, M>,
@@ -36,6 +47,13 @@ pub fn sock_send<M: MemorySize>(
                 .map_err(mem_error_to_wasi)?;
             let iovs_arr = iovs_arr.access().map_err(mem_error_to_wasi)?;
 
+            let nonblocking = fd.flags.contains(Fdflags::NONBLOCK);
+            let timeout = socket
+                .opt_time(TimeType::WriteTimeout)
+                .ok()
+                .flatten()
+                .unwrap_or(Duration::from_secs(30));
+
             let mut sent = 0usize;
             for iovs in iovs_arr.iter() {
                 let buf = WasmPtr::<u8, M>::new(iovs.buf)
@@ -44,7 +62,12 @@ pub fn sock_send<M: MemorySize>(
                     .access()
                     .map_err(mem_error_to_wasi)?;
                 let local_sent = match socket
-                    .send(env.tasks().deref(), buf.as_ref(), fd.flags)
+                    .send(
+                        env.tasks().deref(),
+                        buf.as_ref(),
+                        Some(timeout),
+                        nonblocking,
+                    )
                     .await
                 {
                     Ok(s) => s,
