@@ -23,7 +23,7 @@ pub fn run_wasi(
     filesystem_kind: WasiFileSystemKind,
 ) -> anyhow::Result<()> {
     println!("Running wasi wast `{}`", wast_path);
-    let store = config.store();
+    let mut store = config.store();
 
     let source = {
         let mut out = String::new();
@@ -34,7 +34,7 @@ pub fn run_wasi(
     let tokens = WasiTest::lex_string(&source)?;
     let wasi_test = WasiTest::parse_tokens(&tokens)?;
 
-    let succeeded = wasi_test.run(&store, base_dir, filesystem_kind)?;
+    let succeeded = wasi_test.run(&mut store, base_dir, filesystem_kind)?;
 
     assert!(succeeded);
 
